<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once(BASEPATH.'libraries/Model.php');
require_once(APPPATH.'classes/constants/ConstStatus.interface.php');
require_once(APPPATH.'classes/dataobjects/TransactionSubmitData.class.php');
require_once(APPPATH.'classes/dataobjects/TransactionResponseData.class.php');

/**
 * PaymentDriver クラス
 */
abstract class PaymentDriverBase extends Model
{
	/**
	 * @property PaymentGwMst $_payment_info PaymentGwMst のインスタンス
	 */
	protected $_payment_info = null;
	/**
	 * @property TransactionSubmitData $_submit_object サブミットオブジェクトのインスタンス
	 */
	protected $_submit_object = null;
	/**
	 * @property TransactionResponseData $_response_object レスポンスオブジェクトのインスタンス
	 */
	protected $_response_object = null;
	/**
	 * @property string $_submit_object_name サブミットオブジェクトのクラス名
	 */
	protected $_submit_object_name = 'TransactionSubmitData';
	/**
	 * @property string $_response_object_name レスポンスオブジェクトのクラス名
	 */
	protected $_response_object_name = 'TransactionResponseData';
	/**
	 * @property string $_last_trans_id 最後に実行したＧＷシステムのトランザクションコード
	 */
	protected $_last_trans_id = null;
	/**
	 * @property enum $_last_trans_proc_id 最後に実行したＧＷシステムのトランザクションの段階
	 */
	protected $_last_trans_proc_id = ConstStatus::TRN_UNKNOWN;

	/**
	 * コンストラクタ
	 */
    public function __construct() {}

	/**
	 * sale transaction
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @param string $transid 確認するトランザクションコード。オプション
	 * @return boolean サブミットできれば true。
	 */
	abstract public function transactionSale($paramobj);
   	/**
     * authorization transaction
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @param string $transid 確認するトランザクションコード。オプション
	 * @return boolean サブミットできれば true。
     */
    abstract public function transactionAuthorization($paramobj);
    /**
     * delayed capture transaction
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @param string $transid 確認するトランザクションコード
	 * @return boolean サブミットできれば true。
     */
    abstract public function transactionDelayedCapture($paramobj);
	/**
	 * void transaction
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @param string $transid 確認するトランザクションコード
	 * @return boolean サブミットできれば true。
	 */
    abstract public function transactionVoid($paramobj);
	/**
	 * credit transaction
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @param string $transid 確認するトランザクションコード
	 * @return boolean サブミットできれば true。
	 */
    abstract public function transactionCredit($paramobj);
	/**
	 * Void が可能かどうか確認します
	 * @param TransactionRequestData $paramobj httpリクエスト
	 * @param string $transid 確認するトランザクションコード
	 * @return boolean void が可能なら true。
	 */
    public function isAvailableVoid($paramobj)
	{
		return false;
	}
	/**
	 * 最後に実行したトランザクションリクエストが成功したかどうか
	 * @return boolean 成功していれば true
	 */
    public function isTransactionSucceed()
    {
		$obj_res =& $this->getResponseObject();
		return $obj_res->isSucceed();
    }
    /**
     * インスタンスを初期化します
     * @param PaymentGwMst $payment PaymentGwMst のインスタンス
     */
    public function initialize($payment)
    {
    	$this->_payment_info = $payment;
    }
    /**
     * 決済システムのトランザクションの段階値を返します
     * @return enum 決済システムのトランザクションの段階値
     */
    public function getTransProcId()
    {
    	return $this->_last_trans_proc_id;
    }
    /**
	 * Submit データオブジェクトのインスタンスを返します。
	 * インスタンスが存在しなかった場合は新規に作成します。
	 * @return DataObjectBase $this->_submit_object への参照
     */
    public function& getSubmitObject()
    {
		if(is_null($this->_submit_object))
			$this->_submit_object = new $this->_submit_object_name();
		return $this->_submit_object;
	}
    /**
	 * Response データオブジェクトのインスタンスを返します。
	 * インスタンスが存在しなかった場合は新規に作成します。
	 * @return DataObjectBase $this->_response_object への参照
     */
    public function& getResponseObject()
    {
		if(is_null($this->_response_object))
			$this->_response_object = new $this->_response_object_name();
		return $this->_response_object;
	}
    /**
	 * Submit データオブジェクトの Namevalue 文字列を返します。
	 * @return string Submit データオブジェクトの Namevalue 文字列
     */
    public function getSubmitValueString()
    {
		$obj =& $this->getSubmitObject();
    	return $obj->toNamevalueString();
	}
    /**
	 * Response データオブジェクトの Namevalue 文字列を返します。
	 * @return string Response データオブジェクトの Namevalue 文字列
     */
    public function getResponseValueString()
    {
		$obj =& $this->getResponseObject();
    	return $obj->toNamevalueString();
	}
    /**
     * インスタンスにトランザクションIDをセットします。
     * @param string $transid トランザクションID。省略時は新規に作成します。
     * @return string セット／作成されたトランザクションID
     */
    public function setTransactionId($transid = null)
    {
		if(empty($transid))
			$transid = self::createTransactionId();
    	$this->_last_trans_id = $transid;
    	//
    	return $transid;
    }
    /**
     * インスタンスのトランザクションコードを返します。
     * @return string トランザクションコード
     */
    public function getTransactionId()
    {
    	return $this->_last_trans_id;
    }
    /**
     * 新しいトランザクションコードを作成します。
     * @return string トランザクションコード文字列
     */
    public static function createTransactionId()
    {
    	//$seed = md5(date("r") . mt_rand());
    	$seed = md5(uniqid(mt_rand(), true));
    	return $seed;
    }
    /**
     * 決済システムのオリジナルトランザクションコードを返します。
     * @return string 決済システムのオリジナルトランザクションコード
     */
    public function getOriginalTransactionCode()
    {
		$obj =& $this->getResponseObject();
    	return $obj->getTransactionCode();
    }
    /**
     * 決済システムからのレスポンスをＧＷシステムのリザルトに変換して返します。
     * @param TransactionResultData $result ＧＷシステム用リザルトオブジェクト。省略可
     * @return TransactionResultData ＧＷシステム用リザルトオブジェクト
     */
    public function convertResponseToResult($result = null)
	{
		//
		if(empty($result))
			$result = new TransactionResultData();
		//
		$response = $this->getResponseObject();
		$result->setResultCode($response->getResultCode());
		$result->RESPMSG	= $response->getResultMessage();
		$result->PNREF		= $this->getTransactionId();
		//
		return $result;
	}
}

/* End of file */