<?php

/**
 * データオブジェクト基底クラス
 * @package DataObjectBase
 */
class DataObjectBase implements IteratorAggregate
{
	/**
	 * オー バーロード以外の既定メンバーの名前のリスト。
	 * 連想配列の場合は、要素名でアクセスされた場合に対応する名前のメンバーにアクセスする。
	 * 'ASSIGN_NAME' => 'member' だった場 合、$obj->ASSIGN_NAME、$obj->member どちらでも
	 * $obj->member にアクセスする。
	 *
	 * @abstract
	 * @access protected
	 */
	protected $_propertynames = array();

	/**
	 * オーバーロードされるデータの場所
	 */
	protected $_members = array();

	/**
	 * コンストラクタ
	 *
	 * @param array $datas メンバー名=>値 形式の連想配列
	 */
	public function __construct($datas = array())
	{
		//
//		$this->_initPropertyNames();
		// プロパティリストからデータメンバーを初期化
		$this->emptyObject();
		// メンバーに値設定
		if(!empty($datas))
			$this->parseArray($datas);
	}
	/**
	 * implements IteratorAggregate::getIterator
	 *
	 * @access public
	 * @return Iterator
	 */
    public function getIterator()
    {
        return new ArrayIterator($this->_members);
    }
	/**
	 * プロパティリストからデータメンバーを初期化します
	 *
	 * @return void
	 */
	public function emptyObject()
	{
		$this->_members = array();
		foreach($this->_propertynames as $property => $name)
		{
			$this->$name = null;
		}
	}

//	/**
//	 *
//	 */
//	protected function _initPropertyNames()
//	{
//		$propertynames = array();
//		// 要素名＝指定名 として $fieldassign に基づき取得
//		foreach($this->_propertynames as $property => $name)
//		{
//			// $property が数値ならただの配列としてメンバー名で取得
//			if(is_int($property))
//				$propertynames[$name] = $name;
//			else
//				$propertynames[$property] = $name;
//		}
//		//
//		$this->_propertynames = $propertynames;
//	}

	/**
	 * メンバー名前リストに指定の名前が存在する場合は、それに該当するメンバーの名前を返す
	 *
	 * @param string $name メンバー名前リストの指定の名前
	 * @return string 該当するメンバーの名前
	 */
	public function getPropertyName($name)
	{
	    return array_key_exists($name, $this->_propertynames)? $this->_propertynames[$name] : $name;
	}
	/**
	 * メンバー名前リストを返す。
	 *
	 * @return array メンバー名前リストの配列
	 */
	public function getPropertyNames()
	{
	    return $this->_propertynames;
	}
	/**
	 * オーバーロード：メンバーセッター
	 *
	 * @access public
	 * @param string $name メンバーの名前
	 * @param mixed $value メンバーの値
	 * @return void
	 */
    public function __set($name, $value)
    {
        // プロパティリストからメンバー名を検索
        $name = $this->getPropertyName($name);
        // 値をセット
        $this->_members[$name] = $value;
    }
	/**
	 * オーバーロード：メンバーゲッター
	 *
	 * @access public
	 * @param string $name メンバーの名前
	 * @return mixed メンバーの値
	 */
	public function __get($name)
	{
        // プロパティリストからメンバー名を検索
        $name = $this->getPropertyName($name);
        // 値を返す
	    return array_key_exists($name, $this->_members)? $this->_members[$name] : null;
	}
	/**
	 * オーバーロード：変数がセットされているかどうかを検査する
	 *
	 * @access public
	 * @param string $name メンバーの名前
	 * @return bool セットされていれば true
	 */
	public function __isset($name)
	{
        // プロパティリストからメンバー名を検索
        $name = $this->getPropertyName($name);
        // 値を検査
	    return isset($this->_members[$name]);
	}
	/**
	 * オーバーロード：指定した変数の割当を解除する
	 *
	 * @access public
	 * @param string $name メンバーの名前
	 * @return void
	 */
	public function __unset($name)
	{
        // プロパティリストからメンバー名を検索
        $name = $this->getPropertyName($name);
        // 値を解除
	    unset($this->_members[$name]);
	}
	/**
	 * オーバーロード：未定義メソッド
	 *
	 * @access public
	 * @param string $name メソッドの名前
	 * @param array $arguments 引数の配列
	 * @return mixed
	 */
    public function __call($name, $arguments)
    {
		$matches = null;
		/*
		 * toString_xxxx
		 */
        // メソッド呼び出し
		if(preg_match('/^toString_(.+)/' , $name, $matches))
			return $this->_toStringOverload($matches[1]);

		/*
		 * TODO2: 吐け Call to undefined method
		 */
    }
    /**
     * toString メソッドが定義されていないメンバーの代理メソッド
     *
     * @access protected
	 * @param string $name メンバーの名前
	 * @return string 文字列化されたメンバーの値
     */
    protected function _toStringOverload($name)
    {
        // プロパティリストからメンバー名を検索
        $name = $this->getPropertyName($name);
        // メンバーが存在すれば文字列化して返す
    	return isset($this->$name)? (string)$this->$name : null;
    }
	/**
	 * 連想配列からメンバを初期化します。
	 * $fieldassign が指定された場合は、
	 * 連想配列なら指定名=>メンバー名で値を割り当て、
	 * ただの配列ならメンバー名で値を割り当てます。
	 * $fieldassign に含まれないメンバーは $datas に含まれていても設定されません。
	 *
	 * @param array $datas メンバー名=>値 形式の連想配列
	 * @param array $fieldassign 指定名=>メンバー名 形式の連想配列、またはメンバー名の配列
	 * @return void
	 */
	public function parseArray($datas, $fieldassign = array())
	{
		if(empty($fieldassign))
		{
			// 要素名＝メンバー名 で設定
			foreach($datas as $name => $value)
				$this->$name = $value;
		}
		else
		{
			// 要素名＝指定名 として $fieldassign に基づき設定
			foreach($fieldassign as $property => $name)
			{
				// $property が数値ならただの配列としてメンバー名で割り当てる
				if(is_int($property))
					$this->$name = @$datas[$name];
				else
					$this->$name = @$datas[$property];

			}
		}
	}
	/**
	 * メンバを配列で返します
	 * $fieldassign が指定された場合は、
	 * 連想配列なら指定名=>メンバー名で値を取得し、
	 * ただの配列ならメンバー名で値を取得し、
	 * true なら $this->_propertynames を使用して値を取得します。
	 * $fieldassign に含まれないメンバーは返されません。
	 *
	 * @access public
	 * @param array $fieldassign 指定名=>メンバー名 形式の連想配列、またはメンバー名の配列
	 * @return array メンバー|指定名=>値の連想配列
	 */
	public function toArray($fieldassign = null)
	{
		$res = array();

		// メンバー代理名で返すための設定
		if($fieldassign === true)
			$fieldassign = $this->_propertynames;

		//
		if(is_array($fieldassign) && !empty($fieldassign))
		{
			// 要素名＝指定名 として $fieldassign に基づき取得
			foreach($fieldassign as $property => $name)
			{
				// $property が数値ならただの配列としてメンバー名で取得
				if(is_int($property))
					$res[$name] = $this->$name;
				else
					$res[$property] = $this->$name;
			}
		}
		else
		{
			// 要素名＝メンバー名 でそのまま返す
			$res = $this->_members;
		}
		//
		return $res;
	}
	/**
	 * メンバ値を文字列に変換した配列を返します。
	 * $fieldassign が指定された場合は、
	 * 連想配列なら指定名=>メンバー名で値を取得し、
	 * ただの配列ならメンバー名で値を取得し、
	 * true なら $this->_propertynames を使用して値を取得します。
	 * $fieldassign に含まれないメンバーは返されません。
	 *
	 * @access public
	 * @param array $fieldassign 指定名=>メンバー名 形式の連想配列、またはメンバー名の配列
	 * @param bool $inclempty 空のフィールドも含む場合は true。デフォルトは false
	 * @return array メンバー|指定名=>文字列値の連想配列
	 */
	public function toStringArray($fieldassign = null, $inclempty = false)
	{
		$res = array();

		// メンバー代理名で返すための設定
		if($fieldassign === true)
			$fieldassign = $this->_propertynames;

		//
		if(is_array($fieldassign) && !empty($fieldassign))
		{
			// 要素名＝指定名 として $fieldassign に基づき取得
			foreach($fieldassign as $property => $name)
			{
	   			$method = 'toString_' . $name;
				// 値取得
				$val = $this->$method();
				// 空フィールド除外の時は continue
				if(!$inclempty && !is_numeric($val) && $val == '')
					continue;
				// $property が数値ならただの配列としてメンバー名で取得
				if(is_int($property))
					$res[$name] = $val;
				else
					$res[$property] = $val;
			}
		}
		else
		{
			// 要素名＝メンバー名 で取得
			foreach($this->_members as $name => $value)
			{
	   			$method = 'toString_' . $name;
				// 値取得
				$val = $this->$method();
				// 空フィールド除外の時は continue
				if(!$inclempty && !is_numeric($val) && $val == '')
					continue;
				// 値設定
				$res[$name] = $val;
			}
		}
		//
		return $res;
	}
	/**
	 * メンバーから Namevalue 文字列を作成して返します。
	 *
	 * @param array $fieldassign 指定名=>メンバー名 形式の連想配列、またはメンバー名の配列
	 * @param bool $inclempty 空のフィールドも含む場合は true。デフォルトは false
	 * @return string Namevalue 文字列
	 */
    public function toNamevalueString($fieldassign = null, $inclempty = false)
    {
    	$params = $this->toStringArray($fieldassign);
    	$values = array();
    	foreach($params as $key => $value)
    	{
    		//
    		$key = rawurlencode($key);
    		$value = rawurlencode($value);
    		//
			if(!$inclempty && !is_numeric($value) && $value == '')
				continue;
    		$values[] = "$key=$value";
    	}
    	return join('&', $values);
    }
	/**
	 * Namevalue 文字列からメンバーに値をセットします。
	 *
	 * @param string $namevalue Namevalue 文字列
	 * @param array $fieldassign 指定名=>メンバー名 形式の連想配列、またはメンバー名の配列
	 * @return void
	 */
    public function parseNamevalueString($namevalue, $fieldassign = array())
    {
    	$datas = array();
    	//
    	$values = explode('&', $namevalue);
    	foreach($values as $value)
    	{
    		@list($key, $val) = explode('=', $value);
    		$datas[$key] = urldecode($val);
    	}
    	//
    	$this->parseArray($datas, $fieldassign);
    }
}

/* End of file */
